@@ -1,28 +1,25 @@
 from typing import Generator
 from unittest import mock
 
-from tests.factories import AttrFactory
-from tests.factories import ClassFactory
-from tests.factories import FactoryTestCase
-from xsdata.codegen.mappers.definitions import DefinitionsMapper
-from xsdata.codegen.models import Class
-from xsdata.codegen.models import Status
+from xsdata.codegen.mappers import DefinitionsMapper
+from xsdata.codegen.models import Class, Status
 from xsdata.formats.dataclass.models.generics import AnyElement
-from xsdata.models.enums import DataType
-from xsdata.models.enums import Namespace
-from xsdata.models.wsdl import Binding
-from xsdata.models.wsdl import BindingMessage
-from xsdata.models.wsdl import BindingOperation
-from xsdata.models.wsdl import Definitions
-from xsdata.models.wsdl import Message
-from xsdata.models.wsdl import Part
-from xsdata.models.wsdl import PortType
-from xsdata.models.wsdl import PortTypeMessage
-from xsdata.models.wsdl import PortTypeOperation
-from xsdata.models.wsdl import Service
-from xsdata.models.wsdl import ServicePort
-from xsdata.models.xsd import Element
+from xsdata.models.enums import DataType, Namespace, Tag
+from xsdata.models.wsdl import (
+    Binding,
+    BindingMessage,
+    BindingOperation,
+    Definitions,
+    Message,
+    Part,
+    PortType,
+    PortTypeMessage,
+    PortTypeOperation,
+    Service,
+    ServicePort,
+)
 from xsdata.utils.namespaces import build_qname
+from xsdata.utils.testing import AttrFactory, ClassFactory, FactoryTestCase
 
 
 def mock_create_inner(target: Class, name: str):
@@ -158,8 +155,8 @@ class DefinitionsMapperTests(FactoryTestCase):
     def test_map_binding_operation(
         self, mock_operation_namespace, mock_map_binding_operation_messages
     ):
-        definitions = Definitions(location="foo.wsdl", target_namespace="xsdata")
-        operation = BindingOperation(name="Add")
+        definitions = Definitions(target_namespace="xsdata")
+        operation = BindingOperation(name="Add", location="foo.wsdl")
         operation.ns_map["foo"] = "bar"
         port_operation = PortTypeOperation()
         config = {"a": "one", "b": "two", "style": "rpc"}
@@ -170,10 +167,10 @@ class DefinitionsMapperTests(FactoryTestCase):
         other = ClassFactory.create()
         service = ClassFactory.create(
             qname=build_qname("xsdata", "Calc_Add"),
-            status=Status.PROCESSED,
-            type=BindingOperation,
-            module="foo",
-            package=None,
+            status=Status.FLATTENED,
+            tag=Tag.BINDING_OPERATION,
+            location="foo.wsdl",
+            module=None,
             ns_map={"foo": "bar"},
             attrs=[
                 DefinitionsMapper.build_attr(
@@ -252,6 +249,7 @@ class DefinitionsMapperTests(FactoryTestCase):
                     f"{name}_input",
                     style,
                     namespace,
+                    None,
                 ),
                 mock.call(
                     definitions,
@@ -260,6 +258,7 @@ class DefinitionsMapperTests(FactoryTestCase):
                     f"{name}_input",
                     style,
                     namespace,
+                    None,
                 ),
                 mock.call(
                     definitions,
@@ -268,6 +267,7 @@ class DefinitionsMapperTests(FactoryTestCase):
                     f"{name}_output",
                     style,
                     namespace,
+                    None,
                 ),
             ]
         )
@@ -293,6 +293,7 @@ class DefinitionsMapperTests(FactoryTestCase):
         mock_build_envelope_class.return_value = target
 
         operation.input = BindingMessage()
+        operation.name = "Add"
         port_operation.input = PortTypeMessage()
 
         result = DefinitionsMapper.map_binding_operation_messages(
@@ -311,6 +312,7 @@ class DefinitionsMapperTests(FactoryTestCase):
             f"{name}_input",
             style,
             namespace,
+            "Add",
         )
 
     @mock.patch.object(DefinitionsMapper, "map_port_type_message")
@@ -328,26 +330,35 @@ class DefinitionsMapperTests(FactoryTestCase):
         name = "some_operation_bindings"
         style = "document"
         namespace = "xsdata"
-        definitions = Definitions(location="foo.wsdl", target_namespace="bar")
-        port_type_message = PortTypeMessage(message="some_operation")
+        definitions = Definitions(target_namespace="bar")
+        port_type_message = PortTypeMessage(
+            message="some_operation", location="foo.wsdl"
+        )
         binding_message = BindingMessage(
             extended=[
                 AnyElement(qname="body"),
                 AnyElement(qname="header"),
                 AnyElement(qname="header"),
-            ]
+            ],
+            location="foo.wsdl",
         )
         binding_message.ns_map["foo"] = "bar"
 
         result = DefinitionsMapper.build_envelope_class(
-            definitions, binding_message, port_type_message, name, style, namespace
+            definitions,
+            binding_message,
+            port_type_message,
+            name,
+            style,
+            namespace,
+            None,
         )
 
         expected = Class(
             qname=build_qname("bar", name),
             meta_name="Envelope",
-            type=BindingMessage,
-            module="foo",
+            tag=Tag.BINDING_MESSAGE,
+            location="foo.wsdl",
             ns_map={"foo": "bar"},
             namespace="xsdata",
         )
@@ -404,26 +415,35 @@ class DefinitionsMapperTests(FactoryTestCase):
         name = "some_operation_bindings"
         style = "rpc"
         namespace = "xsdata"
-        definitions = Definitions(location="foo.wsdl", target_namespace="bar")
-        port_type_message = PortTypeMessage(message="some_operation")
+        definitions = Definitions(target_namespace="bar")
+        port_type_message = PortTypeMessage(
+            message="some_operation", location="foo.wsdl"
+        )
         binding_message = BindingMessage(
             extended=[
                 AnyElement(qname="body", attributes={"namespace": "bodyns"}),
                 AnyElement(qname="header"),
                 AnyElement(qname="header"),
-            ]
+            ],
+            location="foo.wsdl",
         )
         binding_message.ns_map["foo"] = "bar"
 
         result = DefinitionsMapper.build_envelope_class(
-            definitions, binding_message, port_type_message, name, style, namespace
+            definitions,
+            binding_message,
+            port_type_message,
+            name,
+            style,
+            namespace,
+            None,
         )
 
         expected = Class(
             qname=build_qname("bar", name),
             meta_name="Envelope",
-            type=BindingMessage,
-            module="foo",
+            tag=Tag.BINDING_MESSAGE,
+            location="foo.wsdl",
             ns_map={"foo": "bar"},
             namespace="xsdata",
         )
@@ -431,7 +451,9 @@ class DefinitionsMapperTests(FactoryTestCase):
         self.assertEqual(1, len(result.inner[0].attrs))
         self.assertEqual(2, len(result.inner[1].attrs))
         self.maxDiff = None
-        mock_map_port_type_message.assert_called_once_with(port_type_message, "bodyns")
+        mock_map_port_type_message.assert_called_once_with(
+            None, port_type_message, "bodyns"
+        )
 
         mock_map_binding_message_parts.assert_has_calls(
             [
@@ -675,7 +697,7 @@ class DefinitionsMapperTests(FactoryTestCase):
                 "arg0", str(DataType.STRING), namespace="", native=True
             ),
             DefinitionsMapper.build_attr(
-                "arg1", build_qname("boo", "cafe"), namespace="", native=False
+                "arg1", build_qname("boo", "cafe"), namespace="##lazy", native=False
             ),
         ]
         self.assertIsInstance(result, Generator)
@@ -686,34 +708,34 @@ class DefinitionsMapperTests(FactoryTestCase):
     def test_build_message_class(self, mock_create_message_attributes):
         message = Message(name="bar", parts=[Part()])
         message.ns_map["foo"] = "bar"
-        definitions = Definitions(
-            messages=[message], target_namespace="xsdata", location="foo.wsdl"
-        )
-        port_type_message = PortTypeMessage(message="foo:bar")
+        definitions = Definitions(messages=[message], target_namespace="xsdata")
+        port_type_message = PortTypeMessage(message="foo:bar", location="foo.wsdl")
 
         attrs = AttrFactory.list(2)
         mock_create_message_attributes.return_value = attrs
         actual = DefinitionsMapper.build_message_class(definitions, port_type_message)
         expected = Class(
-            qname=build_qname("xsdata", "bar"),
-            status=Status.PROCESSED,
-            type=Element,
-            module="foo",
+            qname=build_qname("bar", "bar"),
+            status=Status.RAW,
+            tag=Tag.ELEMENT,
+            location="foo.wsdl",
+            namespace="bar",
             ns_map=message.ns_map,
             attrs=attrs,
         )
         self.assertEqual(expected, actual)
 
     def test_get_or_create_inner_class(self):
-
-        target = ClassFactory.create(module="foo", package=None, ns_map={"foo": "bar"})
+        target = ClassFactory.create(
+            location="foo", package=None, ns_map={"foo": "bar"}
+        )
 
         actual = DefinitionsMapper.build_inner_class(target, "body")
         expected = ClassFactory.create(
-            qname="body",
-            type=BindingMessage,
-            module=target.module,
-            package=None,
+            qname="{xsdata}body",
+            tag=Tag.BINDING_MESSAGE,
+            location=target.location,
+            module=None,
             ns_map=target.ns_map,
         )
         self.assertEqual(expected, actual)
@@ -733,7 +755,7 @@ class DefinitionsMapperTests(FactoryTestCase):
         target_namespace = "xsdata"
 
         actual = DefinitionsMapper.map_port_type_message(
-            port_type_message, target_namespace
+            None, port_type_message, target_namespace
         )
         expected = DefinitionsMapper.build_attr(
             "bar", qname=build_qname("foobar", "bar"), namespace=target_namespace
@@ -742,8 +764,17 @@ class DefinitionsMapperTests(FactoryTestCase):
         self.assertIsInstance(actual, Generator)
         self.assertEqual([expected], list(actual))
 
-    def test_operation_namespace(self):
+        actual = DefinitionsMapper.map_port_type_message(
+            "Add", port_type_message, target_namespace
+        )
+        expected = DefinitionsMapper.build_attr(
+            "Add", qname=build_qname("foobar", "bar"), namespace=target_namespace
+        )
 
+        self.assertIsInstance(actual, Generator)
+        self.assertEqual([expected], list(actual))
+
+    def test_operation_namespace(self):
         self.assertIsNone(DefinitionsMapper.operation_namespace({}))
         self.assertIsNone(DefinitionsMapper.operation_namespace({"transport": "foo"}))
         self.assertEqual(
