@@ -2,35 +2,37 @@ from types import GeneratorType
 from typing import Iterator
 from unittest import mock
 
-from tests.factories import AttrFactory
-from tests.factories import AttrTypeFactory
-from tests.factories import ClassFactory
-from tests.factories import ExtensionFactory
-from tests.factories import FactoryTestCase
-from xsdata.codegen.mappers.schema import SchemaMapper
-from xsdata.codegen.models import Class
-from xsdata.codegen.models import Restrictions
-from xsdata.models.enums import FormType
-from xsdata.models.enums import Tag
-from xsdata.models.xsd import Alternative
-from xsdata.models.xsd import Annotation
-from xsdata.models.xsd import Attribute
-from xsdata.models.xsd import AttributeGroup
-from xsdata.models.xsd import Choice
-from xsdata.models.xsd import ComplexContent
-from xsdata.models.xsd import ComplexType
-from xsdata.models.xsd import Element
-from xsdata.models.xsd import Enumeration
-from xsdata.models.xsd import Extension
-from xsdata.models.xsd import Group
-from xsdata.models.xsd import Override
-from xsdata.models.xsd import Redefine
-from xsdata.models.xsd import Restriction
-from xsdata.models.xsd import Schema
-from xsdata.models.xsd import Sequence
-from xsdata.models.xsd import SimpleContent
-from xsdata.models.xsd import SimpleType
+from xsdata.codegen.mappers import SchemaMapper
+from xsdata.codegen.models import Class, Restrictions
+from xsdata.models.enums import DataType, FormType, Tag
+from xsdata.models.xsd import (
+    Alternative,
+    Annotation,
+    Attribute,
+    AttributeGroup,
+    Choice,
+    ComplexContent,
+    ComplexType,
+    Element,
+    Enumeration,
+    Extension,
+    Group,
+    Override,
+    Redefine,
+    Restriction,
+    Schema,
+    Sequence,
+    SimpleContent,
+    SimpleType,
+)
 from xsdata.utils.namespaces import build_qname
+from xsdata.utils.testing import (
+    AttrFactory,
+    AttrTypeFactory,
+    ClassFactory,
+    ExtensionFactory,
+    FactoryTestCase,
+)
 
 
 class SchemaMapperTests(FactoryTestCase):
@@ -112,6 +114,15 @@ class SchemaMapperTests(FactoryTestCase):
         attribute.target_namespace = "tns"
         self.assertEqual("tns", SchemaMapper.element_namespace(attribute, target_ns))
 
+        element = Element(ref="something")
+        self.assertEqual("foobar", SchemaMapper.element_namespace(element, target_ns))
+
+        element.ns_map["foo"] = "foobar"
+        self.assertEqual("", SchemaMapper.element_namespace(element, target_ns))
+
+        element.ns_map[None] = "foobar"
+        self.assertEqual("foobar", SchemaMapper.element_namespace(element, target_ns))
+
     @mock.patch.object(SchemaMapper, "element_namespace")
     @mock.patch.object(SchemaMapper, "build_class_attributes")
     @mock.patch.object(SchemaMapper, "build_class_extensions")
@@ -140,7 +151,7 @@ class SchemaMapperTests(FactoryTestCase):
 
         element = Element()
         element.ns_map["sm"] = "sm_ns"
-        result = SchemaMapper.build_class(element, "container", "module", "target_ns")
+        result = SchemaMapper.build_class(element, "container", "tests", "target_ns")
 
         mock_build_class_attributes.assert_called_once_with(element, result)
         mock_build_class_extensions.assert_called_once_with(element, result)
@@ -148,14 +159,14 @@ class SchemaMapperTests(FactoryTestCase):
 
         expected = ClassFactory.create(
             qname=build_qname("target_ns", "name"),
-            type=Element,
+            tag=Tag.ELEMENT,
             help="sos",
             abstract=True,
             nillable=True,
             namespace="foo:name",
             ns_map=element.ns_map,
-            package=None,
-            module="module",
+            module=None,
+            location="tests",
             substitutions=[
                 build_qname("target_ns", "foo"),
                 build_qname("sm_ns", "bar"),
@@ -168,15 +179,16 @@ class SchemaMapperTests(FactoryTestCase):
     def test_build_class_extensions(self, mock_children_extensions):
         bar_type = AttrTypeFactory.create(qname="bar")
         foo_type = AttrTypeFactory.create(qname="foo")
-        some_type = AttrTypeFactory.create(qname="{xsdata}something")
 
-        bar = ExtensionFactory.create(type=bar_type)
-        double = ExtensionFactory.create(type=bar_type)
-        foo = ExtensionFactory.create(type=foo_type)
+        bar = ExtensionFactory.create(bar_type, tag=Tag.RESTRICTION)
+        double = ExtensionFactory.create(bar_type, tag=Tag.RESTRICTION)
+        foo = ExtensionFactory.create(foo_type, tag=Tag.EXTENSION)
 
         mock_children_extensions.return_value = [bar, double, foo]
-        self_ext = ExtensionFactory.create(
-            type=some_type, restrictions=Restrictions(min_occurs=1, max_occurs=1)
+        self_ext = ExtensionFactory.reference(
+            qname="{xsdata}something",
+            tag=Tag.ELEMENT,
+            restrictions=Restrictions(min_occurs=1, max_occurs=1),
         )
 
         item = ClassFactory.create()
@@ -188,7 +200,9 @@ class SchemaMapperTests(FactoryTestCase):
 
     def test_element_children(self):
         sequence_one = Sequence(elements=[Element(), Element()])
+        sequence_one.index = 1
         sequence_two = Sequence(max_occurs=2, elements=[Element(), Element()])
+        sequence_two.index = 2
         restriction = Restriction(
             enumerations=[Enumeration(value=x) for x in "abc"], sequence=sequence_two
         )
@@ -219,12 +233,13 @@ class SchemaMapperTests(FactoryTestCase):
         complex_type = ComplexType(
             sequence=Sequence(choices=[choice], min_occurs=0, max_occurs=3)
         )
+        complex_type.sequence.index = 2
         parent_restrictions = Restrictions.from_element(complex_type)
         children = SchemaMapper.element_children(complex_type, parent_restrictions)
 
         child, restrictions = next(children)
         expected = Restrictions(
-            min_occurs=0, max_occurs=3, sequential=True, choice=str(id(choice))
+            path=[("s", id(complex_type.sequence), 0, 3), ("c", id(choice), 1, 1)]
         )
         self.assertEqual(expected, restrictions)
 
@@ -239,20 +254,21 @@ class SchemaMapperTests(FactoryTestCase):
 
         item = ClassFactory.create(ns_map={"bk": "book"})
         children = SchemaMapper.children_extensions(complex_type, item)
-        expected = list(
-            map(
-                ExtensionFactory.create,
-                [
-                    AttrTypeFactory.create(qname=build_qname("book", "b")),
-                    AttrTypeFactory.create(qname=build_qname("book", "c")),
-                ],
-            )
-        )
+        expected = [
+            ExtensionFactory.create(
+                AttrTypeFactory.create(qname=build_qname("book", "b")),
+                tag=Tag.RESTRICTION,
+            ),
+            ExtensionFactory.create(
+                AttrTypeFactory.create(qname=build_qname("book", "c")),
+                tag=Tag.EXTENSION,
+            ),
+        ]
 
         self.assertIsInstance(children, GeneratorType)
         self.assertEqual(expected, list(children))
 
-    @mock.patch.object(SchemaMapper, "build_class_attribute_types")
+    @mock.patch.object(SchemaMapper, "build_attr_types")
     @mock.patch.object(SchemaMapper, "element_namespace")
     @mock.patch.object(Attribute, "get_restrictions")
     @mock.patch.object(Attribute, "is_fixed", new_callable=mock.PropertyMock)
@@ -269,20 +285,18 @@ class SchemaMapperTests(FactoryTestCase):
         mock_is_fixed,
         mock_get_restrictions,
         mock_element_namespace,
-        mock_build_class_attribute_types,
+        mock_build_attr_types,
     ):
         item = ClassFactory.create(ns_map={"bar": "foo"})
 
-        mock_build_class_attribute_types.return_value = AttrTypeFactory.list(
-            1, qname="int"
-        )
+        mock_build_attr_types.return_value = AttrTypeFactory.list(1, qname="int")
         mock_real_name.return_value = item.name
         mock_display_help.return_value = "sos"
         mock_prefix.return_value = "com"
         mock_default_value.return_value = "default"
         mock_is_fixed.return_value = True
         mock_element_namespace.return_value = "http://something/common"
-        mock_get_restrictions.return_value = {"required": True}
+        mock_get_restrictions.return_value = {"min_occurs": 1, "max_occurs": 1}
 
         attribute = Attribute(default="false")
         attribute.index = 66
@@ -291,76 +305,75 @@ class SchemaMapperTests(FactoryTestCase):
         SchemaMapper.build_class_attribute(item, attribute, Restrictions())
         expected = AttrFactory.create(
             name=mock_real_name.return_value,
-            types=mock_build_class_attribute_types.return_value,
+            types=mock_build_attr_types.return_value,
             tag=Tag.ATTRIBUTE,
             namespace=mock_element_namespace.return_value,
             help=mock_display_help.return_value,
             default=mock_default_value.return_value,
             fixed=mock_is_fixed.return_value,
             index=66,
-            restrictions=Restrictions(required=True),
+            restrictions=Restrictions(min_occurs=1, max_occurs=1),
         )
         self.assertEqual(expected, item.attrs[0])
         self.assertEqual({"bar": "foo", "foo": "bar"}, item.ns_map)
-        mock_build_class_attribute_types.assert_called_once_with(item, attribute)
+        mock_build_attr_types.assert_called_once_with(item, attribute)
         mock_element_namespace.assert_called_once_with(attribute, item.target_namespace)
 
-    @mock.patch.object(Attribute, "real_type", new_callable=mock.PropertyMock)
+    @mock.patch.object(Attribute, "attr_types", new_callable=mock.PropertyMock)
     @mock.patch.object(SchemaMapper, "build_inner_classes")
-    def test_build_class_attribute_types(
-        self, mock_build_inner_classes, mock_real_type
-    ):
-        mock_real_type.return_value = " xs:integer  xs:string "
+    def test_build_attr_types(self, mock_build_inner_classes, mock_attr_types):
+        mock_attr_types.return_value = ["xs:integer", "xs:string"]
         mock_build_inner_classes.return_value = []
 
         item = ClassFactory.create()
         attribute = Attribute(default="false")
-        actual = SchemaMapper.build_class_attribute_types(item, attribute)
+        actual = SchemaMapper.build_attr_types(item, attribute)
 
-        expected = [AttrTypeFactory.xs_int(), AttrTypeFactory.xs_string()]
+        expected = [
+            AttrTypeFactory.native(DataType.INTEGER),
+            AttrTypeFactory.native(DataType.STRING),
+        ]
 
         self.assertEqual(expected, actual)
 
-    @mock.patch.object(Attribute, "real_type", new_callable=mock.PropertyMock)
+    @mock.patch.object(Attribute, "attr_types", new_callable=mock.PropertyMock)
     @mock.patch.object(SchemaMapper, "build_inner_classes")
-    def test_build_class_attribute_types_when_obj_has_inner_class(
-        self, mock_build_inner_classes, mock_real_type
+    def test_build_attr_types_when_obj_has_inner_class(
+        self, mock_build_inner_classes, mock_attr_types
     ):
         inner_class = ClassFactory.create(qname="foo")
-        mock_real_type.return_value = " xs:integer  xs:string "
+        mock_attr_types.return_value = ["xs:integer", "xs:string"]
         mock_build_inner_classes.return_value = [inner_class]
 
         item = ClassFactory.create()
         attribute = Attribute(default="false")
-        actual = SchemaMapper.build_class_attribute_types(item, attribute)
+        actual = SchemaMapper.build_attr_types(item, attribute)
 
         expected = [
-            AttrTypeFactory.xs_int(),
-            AttrTypeFactory.xs_string(),
-            AttrTypeFactory.create(
-                qname=build_qname(item.target_namespace, "foo"), forward=True
-            ),
+            AttrTypeFactory.native(DataType.INTEGER),
+            AttrTypeFactory.native(DataType.STRING),
+            AttrTypeFactory.create(qname=inner_class.qname, forward=True),
         ]
 
         self.assertEqual(expected, actual)
         self.assertEqual([inner_class], item.inner)
 
     @mock.patch.object(Attribute, "default_type", new_callable=mock.PropertyMock)
-    @mock.patch.object(Attribute, "real_type", new_callable=mock.PropertyMock)
+    @mock.patch.object(Attribute, "attr_types", new_callable=mock.PropertyMock)
     @mock.patch.object(SchemaMapper, "build_inner_classes")
-    def test_build_class_attribute_types_when_obj_has_no_types(
-        self, mock_build_inner_classes, mock_real_type, mock_default_type
+    def test_build_attr_types_when_obj_has_no_types(
+        self, mock_build_inner_classes, mock_attr_types, mock_default_type
     ):
-        mock_real_type.return_value = ""
+        mock_attr_types.return_value = ""
         mock_build_inner_classes.return_value = []
         mock_default_type.return_value = "xs:string"
 
         item = ClassFactory.create()
         attribute = Attribute(default="false", name="attr")
-        actual = SchemaMapper.build_class_attribute_types(item, attribute)
+        actual = SchemaMapper.build_attr_types(item, attribute)
 
         self.assertEqual(1, len(actual))
-        self.assertEqual(AttrTypeFactory.xs_string(), actual[0])
+        self.assertEqual(AttrTypeFactory.native(DataType.STRING), actual[0])
 
     @mock.patch.object(SchemaMapper, "build_class")
     def test_build_inner_classes(self, mock_build_class):
