@@ -1,30 +1,29 @@
-import json
 from pathlib import Path
 from typing import Type
 
 from lxml import etree
 
-from xsdata.formats.dataclass.parsers import JsonParser
-from xsdata.formats.dataclass.parsers import XmlParser
-from xsdata.formats.dataclass.serializers import JsonSerializer
-from xsdata.formats.dataclass.serializers import XmlSerializer
+from xsdata.formats.dataclass.parsers import JsonParser, XmlParser
+from xsdata.formats.dataclass.serializers import JsonSerializer, XmlSerializer
 from xsdata.formats.dataclass.serializers.config import SerializerConfig
 
 
 def validate_bindings(schema: Path, clazz: Type):
     __tracebackhide__ = True
 
-    obj = XmlParser().from_path(schema.with_suffix(".xml"), clazz)
-    actual = JsonSerializer(indent=4).render(obj)
+    sample = schema.parent.joinpath("sample.xml")
+    obj = XmlParser().from_path(sample, clazz)
+    config = SerializerConfig(indent="  ")
+    actual = JsonSerializer(config=config).render(obj)
 
-    expected = schema.with_suffix(".json")
+    expected = sample.with_suffix(".json")
     if expected.exists():
         assert expected.read_text() == actual
         assert obj == JsonParser().from_string(actual, clazz)
     else:
         expected.write_text(actual, encoding="utf-8")
 
-    config = SerializerConfig(pretty_print=True)
+    config = SerializerConfig(indent="  ")
     xml = XmlSerializer(config=config).render(obj)
 
     validator = etree.XMLSchema(etree.parse(str(schema)))
