@@ -1,18 +1,13 @@
 import json
-from decimal import Decimal
 from unittest.case import TestCase
-from xml.etree.ElementTree import QName
 
-from tests.fixtures.books import BookForm
-from tests.fixtures.books import Books
-from xsdata.formats.dataclass.serializers import DictFactory
-from xsdata.formats.dataclass.serializers.json import asdict
-from xsdata.formats.dataclass.serializers.json import JsonEncoder
-from xsdata.formats.dataclass.serializers.json import JsonSerializer
-from xsdata.models.enums import FormType
+from tests.fixtures.books import BookForm, Books
+from xsdata.formats.dataclass.serializers.config import SerializerConfig
+from xsdata.formats.dataclass.serializers.json import DictFactory, JsonSerializer
+from xsdata.models.datatype import XmlDate
 
 
-class JsonEncoderTests(TestCase):
+class JsonSerializerTests(TestCase):
     def setUp(self):
         super().setUp()
         self.books = Books(
@@ -23,7 +18,7 @@ class JsonEncoderTests(TestCase):
                     title="The First Book",
                     genre="Fiction",
                     price=44.95,
-                    pub_date="2000-10-01",
+                    pub_date=XmlDate.from_string("2000-10-01"),
                     review="An amazing story of nothing.",
                 ),
                 BookForm(
@@ -36,11 +31,7 @@ class JsonEncoderTests(TestCase):
             ]
         )
 
-    def test_render(self):
-        serializer = JsonSerializer(dict_factory=DictFactory.FILTER_NONE)
-        actual = serializer.render(self.books)
-
-        expected = {
+        self.expected = {
             "book": [
                 {
                     "author": "Hightower, Kim",
@@ -62,20 +53,41 @@ class JsonEncoderTests(TestCase):
                 },
             ]
         }
-        self.assertEqual(expected, json.loads(actual))
 
-    def test_encode_qname(self):
-        actual = json.dumps({"qname": QName("a", "b")}, cls=JsonEncoder)
-        self.assertEqual('{"qname": "{a}b"}', actual)
+    def test_render(self):
+        serializer = JsonSerializer(dict_factory=DictFactory.FILTER_NONE)
+        actual = serializer.render(self.books)
 
-    def test_encode_enum(self):
-        actual = json.dumps({"enum": FormType.QUALIFIED}, cls=JsonEncoder)
-        self.assertEqual('{"enum": "qualified"}', actual)
+        self.assertEqual(self.expected, json.loads(actual))
 
-    def test_encode_decimal(self):
-        actual = json.dumps({"decimal": Decimal(10.5)}, cls=JsonEncoder)
-        self.assertEqual('{"decimal": "10.5"}', actual)
+    def test_indent(self):
+        config = SerializerConfig(indent="    ")
+        serializer = JsonSerializer(config=config)
 
-    def test_default_raises_exceptions(self):
-        with self.assertRaises(TypeError):
-            json.dumps({"string": object()}, cls=JsonEncoder)
+        actual = serializer.render(self.books)
+        expected = f"""{{
+    "book": [
+        {{
+            "author": "{self.expected["book"][0]["author"]}",
+            "title": "{self.expected["book"][0]["title"]}",
+            "genre": "{self.expected["book"][0]["genre"]}",
+            "price": {self.expected["book"][0]["price"]},
+            "pub_date": "{self.expected["book"][0]["pub_date"]}",
+            "review": "{self.expected["book"][0]["review"]}",
+            "id": "{self.expected["book"][0]["id"]}",
+            "lang": "en"
+        }},
+        {{
+            "author": "{self.expected["book"][1]["author"]}",
+            "title": "{self.expected["book"][1]["title"]}",
+            "genre": "{self.expected["book"][1]["genre"]}",
+            "price": null,
+            "pub_date": null,
+            "review": "{self.expected["book"][1]["review"]}",
+            "id": "{self.expected["book"][1]["id"]}",
+            "lang": "en"
+        }}
+    ]
+}}"""
+
+        self.assertEqual(expected, actual)
